/*
 * LCD peripheral support for the JZ4740 and related SoCs.
 *
 * Copyright (C) Xiangfu Liu <xiangfu@sharism.cc>
 * Copyright (C) 2015, 2016, 2017, 2018, 2020,
 *               2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once



enum Regs : unsigned
{
  Lcd_config          = 0x000,  // LCD_CFG
  Lcd_vsync           = 0x004,  // LCD_VSYNC
  Lcd_hsync           = 0x008,  // LCD_HSYNC
  Virtual_area        = 0x00c,  // LCD_VAT
  Display_hlimits     = 0x010,  // LCD_DAH
  Display_vlimits     = 0x014,  // LCD_DAV
  Lcd_ps              = 0x018,  // LCD_PS
  Lcd_cls             = 0x01c,  // LCD_CLS
  Lcd_spl             = 0x020,  // LCD_SPL
  Lcd_rev             = 0x024,  // LCD_REV
  Lcd_control         = 0x030,  // LCD_CTRL
  Lcd_status          = 0x034,  // LCD_STATE
  Lcd_irq_id          = 0x038,  // LCD_IID
  Desc_address_0      = 0x040,  // LCD_DA0
  Source_address_0    = 0x044,  // LCD_SA0
  Frame_id_0          = 0x048,  // LCD_FID0
  Command_0           = 0x04c,  // LCD_CMD0
  Counter_position_0  = 0x068,  // LCD_CPOS0
  Foreground_size_0   = 0x06c,  // LCD_DESSIZE0
  Desc_address_1      = 0x050,  // LCD_DA1
  Source_address_1    = 0x054,  // LCD_SA1
  Frame_id_1          = 0x058,  // LCD_FID1
  Command_1           = 0x05c,  // LCD_CMD1
  Counter_position_1  = 0x078,  // LCD_CPOS1
  Foreground_size_1   = 0x07c,  // LCD_DESSIZE1
  Rgb_control         = 0x090,  // LCD_RGBC (JZ4780)
  Alpha_levels        = 0x118,  // LCD_ALPHA (JZ4780)
  Priority_level      = 0x2c0,  // LCD_PCFG

  // OSD registers.

  Osd_config          = 0x100,  // LCD_OSDC
  Osd_control         = 0x104,  // LCD_OSDCTRL
  Osd_status          = 0x108,  // LCD_OSDS
};

// Lcd_config descriptions.

enum Config_values : unsigned
{
  Config_stn_pins_mask  = 0x3,
  Config_mode_mask      = 0xf,
};

// Field positions for registers employing two values, with the first typically
// being the start value and the second being an end value.

enum Value_pair_bits : unsigned
{
  Value_first           = 16,
  Value_second          = 0,
};

// Virtual area bits.

enum Virtual_area_values : unsigned
{
  Virtual_area_horizontal_size  = Value_first,  // sum of display and blank regions (dot/pixel clock periods)
  Virtual_area_vertical_size    = Value_second, // sum of display and blank regions (line periods)
};

// Lcd_control descriptions.

enum Control_bits : unsigned
{
  Control_pin_modify                = 31, // PINMD (change pin usage from 15..0 to 17..10, 8..1)
  Control_burst_length              = 28, // BST (burst length selection)
  Control_rgb_mode                  = 27, // RGB (RGB mode)
  Control_out_underrun              = 26, // OFUP (output FIFO underrun protection)
  Control_frc_algorithm             = 24, // FRC (STN FRC algorithm selection)
  Control_palette_delay             = 16, // PDD (load palette delay counter)
  Control_dac_loopback_test         = 14, // DACTE (DAC loopback test)
  Control_frame_end_irq_enable      = 13, // EOFM (end of frame interrupt enable)
  Control_frame_start_irq_enable    = 12, // SOFM (start of frame interrupt enable)
  Control_out_underrun_irq_enable   = 11, // OFUM (output FIFO underrun interrupt enable)
  Control_in0_underrun_irq_enable   = 10, // IFUM0 (input FIFO 0 underrun interrupt enable)
  Control_in1_underrun_irq_enable   = 9,  // IFUM1 (input FIFO 1 underrun interrupt enable)
  Control_disabled_irq_enable       = 8,  // LDDM (LCD disable done interrupt enable)
  Control_quick_disabled_irq_enable = 7,  // QDM (LCD quick disable done interrupt enable)
  Control_endian_select             = 6,  // BEDN (endian selection)
  Control_bit_order                 = 5,  // PEDN (bit order in bytes)
  Control_disable                   = 4,  // DIS (disable controller)
  Control_enable                    = 3,  // ENA (enable controller)
  Control_bpp                       = 0,  // BPP (bits per pixel)
};

enum Burst_length_values : unsigned
{
  Burst_length_4          = 0,  // 4 word
  Burst_length_8          = 1,  // 8 word
  Burst_length_16         = 2,  // 16 word

  // JZ4780 extensions.

  Burst_length_32         = 3,  // 32 word
  Burst_length_64         = 4,  // 64 word
  Burst_length_mask       = 0x7,
};

enum Rgb_mode_values : unsigned
{
  Rgb_mode_565            = 0,
  Rgb_mode_555            = 1,
  Rgb_mode_mask           = 0x1,
};

enum Frc_algorithm_values : unsigned
{
  Frc_greyscales_16       = 0,
  Frc_greyscales_4        = 1,
  Frc_greyscales_2        = 2,
  Frc_greyscales_mask     = 0x3,
};

enum Control_bpp_values : unsigned
{
  Control_bpp_1bpp        = 0,
  Control_bpp_2bpp        = 1,
  Control_bpp_4bpp        = 2,
  Control_bpp_8bpp        = 3,
  Control_bpp_15bpp       = 4,
  Control_bpp_16bpp       = 4,
  Control_bpp_18bpp       = 5,
  Control_bpp_24bpp       = 5,
  Control_bpp_24bpp_comp  = 6,
  Control_bpp_30bpp       = 7,
  Control_bpp_32bpp       = 7,
  Control_bpp_mask        = 0x7,
};

// Command descriptions.

enum Command_bits : unsigned
{
  Command_frame_start_irq   = 31, // SOFINT (start of frame interrupt)
  Command_frame_end_irq     = 30, // EOFINT (end of frame interrupt)
  Command_lcm_command       = 29, // JZ4780: CMD (LCM command/data via DMA0)
  Command_palette_buffer    = 28, // PAL (descriptor references palette, not display data)
  Command_frame_compressed  = 27, // JZ4780: COMPEN (16/24bpp compression enabled)
  Command_frame_enable      = 26, // JZ4780: FRM_EN
  Command_field_even        = 25, // JZ4780: FIELD_SEL (interlace even field)
  Command_16x16_block       = 24, // JZ4780: 16x16BLOCK (fetch data by 16x16 block)
  Command_buffer_length     = 0,  // LEN
};

enum Command_values : unsigned
{
  Command_buffer_length_mask  = 0x00ffffff,
};

// Status descriptions.

enum Status_bits : unsigned
{
  Status_frame_end_irq    = 5,
  Status_frame_start_irq  = 4,
  Status_out_underrun_irq = 3,
  Status_in0_underrun_irq = 2,
  Status_in1_underrun_irq = 1,
  Status_disabled         = 0,
};

// OSD configuration bits (JZ4780).

enum Osd_config_bits : unsigned
{
  Osd_config_fg1_pixel_alpha_enable     = 17,
  Osd_config_fg1_frame_start_irq_enable = 15,
  Osd_config_fg1_frame_end_irq_enable   = 14,
  Osd_config_fg0_frame_start_irq_enable = 11,
  Osd_config_fg0_frame_end_irq_enable   = 10,
  Osd_config_fg1_enable                 = 4,
  Osd_config_fg0_enable                 = 3,
  Osd_config_alpha_enable               = 2,
  Osd_config_fg0_pixel_alpha_enable     = 1,
  Osd_config_enable                     = 0,
};

enum Osd_control_bits : unsigned
{
  Osd_control_ipu_clock_enable          = 15,
};

// RGB control (JZ4780).

enum Rgb_control_bits : unsigned
{
  Rgb_data_padded         = 15, // RGBDM
  Rgb_padding_mode        = 14, // DMM
  Rgb_422                 = 8,  // 422
  Rgb_format_enable       = 7,  // RGBFMT
  Rgb_odd_line            = 4,  // OddRGB
  Rgb_even_line           = 0,  // EvenRGB
};

enum Rgb_control_values : unsigned
{
  Rgb_padding_end         = 0U << Rgb_padding_mode,
  Rgb_padding_start       = 1U << Rgb_padding_mode,
  Rgb_odd_line_rgb        = 0U << Rgb_odd_line,
  Rgb_odd_line_rbg        = 1U << Rgb_odd_line,
  Rgb_odd_line_grb        = 2U << Rgb_odd_line,
  Rgb_odd_line_gbr        = 3U << Rgb_odd_line,
  Rgb_odd_line_brg        = 4U << Rgb_odd_line,
  Rgb_odd_line_bgr        = 5U << Rgb_odd_line,
  Rgb_even_line_rgb       = 0U << Rgb_even_line,
  Rgb_even_line_rbg       = 1U << Rgb_even_line,
  Rgb_even_line_grb       = 2U << Rgb_even_line,
  Rgb_even_line_gbr       = 3U << Rgb_even_line,
  Rgb_even_line_brg       = 4U << Rgb_even_line,
  Rgb_even_line_bgr       = 5U << Rgb_even_line,
};

// Alpha levels (JZ4780).

enum Alpha_levels_bits : unsigned
{
  Alpha_level_fg1         = 8,
  Alpha_level_fg0         = 0,
};

enum Alpha_levels_values : unsigned
{
  Alpha_level_fg1_mask    = 0x0000ff00,
  Alpha_level_fg0_mask    = 0x000000ff,
};

// Priority level.

enum Priority_level_bits : unsigned
{
  Priority_mode           = 31,
  Priority_highest_burst  = 28,
  Priority_threshold2     = 18,
  Priority_threshold1     = 9,
  Priority_threshold0     = 0,
};

enum Priority_level_values : unsigned
{
  Priority_mode_dynamic   = 0U << Priority_mode,
  Priority_mode_arbiter   = 1U << Priority_mode,
};

enum Priority_burst_values : unsigned
{
  Priority_burst_4        = 0,
  Priority_burst_8        = 1,
  Priority_burst_16       = 2,
  Priority_burst_32       = 3,
  Priority_burst_64       = 4,
  Priority_burst_16_cont  = 5,
  Priority_burst_disable  = 7,
};

// Position descriptor member.

enum Position_bits : unsigned
{
  Position_alpha_mode       = 31,
  Position_rgb              = 30,
  Position_bpp              = 27,
  Position_premultiply_lcd  = 26,
  Position_coefficient      = 24,
  Position_y_position       = 12,
  Position_x_position       = 0,
};

enum Position_alpha_mode_values : unsigned
{
  Position_alpha_image      = 0,
  Position_alpha_pixel      = 1,
};

enum Position_rgb_values : unsigned
{
  Position_rgb_565          = 0,
  Position_rgb_555          = 1,
};

enum Position_bpp_values : unsigned
{
  Position_bpp_15_16bpp   = 4,
  Position_bpp_18_24bpp   = 5,
  Position_bpp_30bpp      = 7,
};

enum Position_coefficient_values : unsigned
{
  Position_coefficient_0              = 0,
  Position_coefficient_1              = 1,
  Position_coefficient_alpha          = 2,
  Position_coefficient_1_minus_alpha  = 3,
};

// Alpha and size.

enum Alpha_size_bits : unsigned
{
  Alpha_size_alpha  = 24,
  Alpha_size_height = 12,
  Alpha_size_width  = 0,
};
